# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .chat_response import ChatResponse

__all__ = ["ChatListResponse"]

ChatListResponse: TypeAlias = List[ChatResponse]
