# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .conversation_flow_response import ConversationFlowResponse

__all__ = ["ConversationFlowListResponse"]

ConversationFlowListResponse: TypeAlias = List[ConversationFlowResponse]
